@@ -42,7 +42,7 @@ class BaseExpression:
 
     def __bool__(self) -> bool:
         raise TypeError(
-            "Conversion of expression to boolean is not allowed, to prevent accidental use of the logical operators: 'and', 'or', and 'not. "
+            "Conversion of expression to boolean is not allowed, to prevent accidental use of the logical operators: 'and', 'or', and 'not'. "
             "Instead either use the bitwise operators '&', '|' and '~' (but note the difference in operator precedence!), "
             "or the functions 'all_of', 'any_of' and 'not_of'"
         )
@@ -53,10 +53,12 @@ class BaseExpression:
     def get_type(self) -> '_StringExpression':
         return _StringExpression(KQL(f'gettype({self.kql})'))
 
-    def __hash__(self) -> '_StringExpression':
+    def __hash__(self, mod: Union['_NumberExpression', int] = None) -> '_StringExpression':
         """
         https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/hashfunction
         """
+        if mod is not None:
+            return _StringExpression(KQL(f'hash({self.kql}, {_to_kql(mod)})'))
         return _StringExpression(KQL(f'hash({self.kql})'))
 
     def hash_sha256(self) -> '_StringExpression':
@@ -95,18 +97,41 @@ class BaseExpression:
     def __ne__(self, other: _ExpressionType) -> '_BooleanExpression':
         return _BooleanExpression.binary_op(self, ' != ', other)
 
-    def is_in(self, other: _DynamicType) -> '_BooleanExpression':
+    def is_in(self, other: _DynamicType, case_sensitive: bool = False) -> '_BooleanExpression':
         """
         https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/scalar-data-types/dynamic#operators-and-functions-over-dynamic-types
         https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/inoperator
         https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/datatypes-string-operators
+        https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/sethaselementfunction
+
+        Best practices (`full list <https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/best-practices>`_):
+
+        * Use `case_sensitive=True` when possible
         """
         if isinstance(other, (List, Tuple)):
             # For a literal array, we can use 'in'
             # The following RHS is the only place where a literal list does not require being surrounded by 'dynamic()'
-            return _BooleanExpression(KQL(f'{self.kql} in ({", ".join(map(_to_kql, other))})'))
+            return _BooleanExpression(KQL(f'{self.kql} {"in" if case_sensitive else "in~"} ({", ".join(map(_to_kql, other))})'))
+        # Otherwise, Kusto does not accept 'in', and we need to use 'set_has_element', which is always case-sensitive
+        assert case_sensitive, "Member test for non-literal array cannot be case insensitive"
+        return _BooleanExpression(KQL(f'set_has_element({_to_kql(other)}, {self.kql})'))
+
+    def not_in(self, other: _DynamicType, case_sensitive: bool = False) -> '_BooleanExpression':
+        """
+        https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/scalar-data-types/dynamic#operators-and-functions-over-dynamic-types
+        https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/inoperator
+        https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/datatypes-string-operators
+
+        Best practices (`full list <https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/best-practices>`_):
+
+        * Use `case_sensitive=True` when possible
+        """
+        if isinstance(other, (List, Tuple)):
+            # For a literal array, we can use 'in'
+            # The following RHS is the only place where a literal list does not require being surrounded by 'dynamic()'
+            return _BooleanExpression(KQL(f'{self.kql} {"!in" if case_sensitive else "!in~"} ({", ".join(map(_to_kql, other))})'))
         # Otherwise, for some reason Kusto does not accept 'in', and we need to use 'contains' as if 'other' was a string
-        return _BooleanExpression.binary_op(other, ' contains ', self)
+        return _BooleanExpression.binary_op(other, ' !contains_cs ' if case_sensitive else ' !contains ', self)
 
     def is_null(self) -> '_BooleanExpression':
         """
@@ -399,9 +424,23 @@ class _StringExpression(BaseExpression):
         return _ArrayExpression(KQL(f'split({self.kql}, {_to_kql(delimiter)}, {_to_kql(requested_index)})'))
 
     def equals(self, other: _StringType, case_sensitive: bool = False) -> _BooleanExpression:
+        """
+        https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/datatypes-string-operators
+
+        Best practices (`full list <https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/best-practices>`_):
+
+        * Use `case_sensitive=True` when possible
+        """
         return _BooleanExpression.binary_op(self, ' == ' if case_sensitive else ' =~ ', other)
 
     def not_equals(self, other: _StringType, case_sensitive: bool = False) -> _BooleanExpression:
+        """
+        https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/datatypes-string-operators
+
+        Best practices (`full list <https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/best-practices>`_):
+
+        * Use `case_sensitive=True` when possible
+        """
         return _BooleanExpression.binary_op(self, ' != ' if case_sensitive else ' !~ ', other)
 
     def matches(self, regex: _StringType) -> '_BooleanExpression':
@@ -413,12 +452,22 @@ class _StringExpression(BaseExpression):
     def contains(self, other: _StringType, case_sensitive: bool = False) -> _BooleanExpression:
         """
         https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/datatypes-string-operators
+
+        Best practices (`full list <https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/best-practices>`_):
+
+        * When looking for full tokens, `has` works better, since it doesn't look for substrings.
+        * Use `case_sensitive=True` when possible
         """
         return _BooleanExpression.binary_op(self, ' contains_cs ' if case_sensitive else ' contains ', other)
 
     def not_contains(self, other: _StringType, case_sensitive: bool = False) -> _BooleanExpression:
         """
         https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/datatypes-string-operators
+
+        Best practices (`full list <https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/best-practices>`_):
+
+        * When looking for full tokens, `not_has` works better, since it doesn't look for substrings.
+        * Use `case_sensitive=True` when possible
         """
         return _BooleanExpression.binary_op(self, ' !contains_cs ' if case_sensitive else ' !contains ', other)
 
@@ -437,12 +486,18 @@ class _StringExpression(BaseExpression):
     def lower(self) -> '_StringExpression':
         """
         https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/tolowerfunction
+
+        Don't use this method for case-insensitive comparisons. Instead use the `equals` method with `case_sensitive=False`
+        (see `best practices <https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/best-practices>`_).
         """
         return _StringExpression(KQL(f'tolower({self.kql})'))
 
     def upper(self) -> '_StringExpression':
         """
         https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/toupperfunction
+
+        Don't use this method for case-insensitive comparisons. Instead use the `equals` method with `case_sensitive=False`
+        (see `best practices <https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/best-practices>`_).
         """
         return _StringExpression(KQL(f'toupper({self.kql})'))
 
@@ -460,6 +515,14 @@ class _StringExpression(BaseExpression):
             f'{self.as_subexpression()} {"has_cs" if case_sensitive else "has"} {_to_kql(exp, True)}'
         ))
 
+    def has_not(self, exp: _StringType, case_sensitive: bool = False) -> '_BooleanExpression':
+        """
+        https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/datatypes-string-operators
+        """
+        return _BooleanExpression(KQL(
+            f'{self.as_subexpression()} {"!has_cs" if case_sensitive else "!has"} {_to_kql(exp, True)}'
+        ))
+
 
 @_plain_expression(_KustoType.DATETIME)
 class _DatetimeExpression(BaseExpression):
