@@ -1,6 +1,9 @@
-from pykusto.expressions import to_kql
-from pykusto.kql_converters import KQL
-from pykusto.type_utils import TypeRegistrar, KustoType
+# noinspection PyProtectedMember
+from pykusto._src.expressions import _to_kql
+# noinspection PyProtectedMember
+from pykusto._src.kql_converters import KQL
+# noinspection PyProtectedMember
+from pykusto._src.type_utils import _TypeRegistrar, _KustoType
 from test.test_base import TestBase
 
 
@@ -15,13 +18,13 @@ class TestUtils(TestBase):
         self.assertEqual(
             'dynamic({"name": "Alan", "age": 21, "address": ["NY", 36], '
             '"pets": ["Libby", "Panda", "]", "["]})',
-            to_kql(test_dict)
+            _to_kql(test_dict)
         )
 
     def test_type_registrar_for_type(self):
-        test_annotation = TypeRegistrar("Test annotation")
+        test_annotation = _TypeRegistrar("Test annotation")
 
-        @test_annotation(KustoType.STRING)
+        @test_annotation(_KustoType.STRING)
         def str_annotated(s: str) -> str:
             return "response to " + s
 
@@ -32,9 +35,9 @@ class TestUtils(TestBase):
         )
 
     def test_type_registrar_for_obj(self):
-        test_annotation = TypeRegistrar("Test annotation")
+        test_annotation = _TypeRegistrar("Test annotation")
 
-        @test_annotation(KustoType.STRING)
+        @test_annotation(_KustoType.STRING)
         def str_annotated(s: str) -> str:
             return "response to " + s
 
@@ -44,9 +47,9 @@ class TestUtils(TestBase):
         )
 
     def test_type_registrar_for_type_not_found(self):
-        test_annotation = TypeRegistrar("Test annotation")
+        test_annotation = _TypeRegistrar("Test annotation")
 
-        @test_annotation(KustoType.STRING)
+        @test_annotation(_KustoType.STRING)
         def str_annotated(s: str) -> str:
             return "response to " + s
 
@@ -57,9 +60,9 @@ class TestUtils(TestBase):
         )
 
     def test_type_registrar_for_obj_not_found(self):
-        test_annotation = TypeRegistrar("Test annotation")
+        test_annotation = _TypeRegistrar("Test annotation")
 
-        @test_annotation(KustoType.STRING)
+        @test_annotation(_KustoType.STRING)
         def str_annotated(s: str) -> str:
             return "response to " + s
 
@@ -69,9 +72,9 @@ class TestUtils(TestBase):
         )
 
     def test_type_registrar_collision(self):
-        test_annotation = TypeRegistrar("Test annotation")
+        test_annotation = _TypeRegistrar("Test annotation")
 
-        @test_annotation(KustoType.STRING)
+        @test_annotation(_KustoType.STRING)
         def str_annotated_1(s: str) -> KQL:
             return KQL("response to " + s)
 
@@ -80,5 +83,5 @@ class TestUtils(TestBase):
 
         self.assertRaises(
             TypeError("Test annotation: type already registered: string"),
-            lambda: test_annotation(KustoType.STRING)(str_annotated_2)
+            lambda: test_annotation(_KustoType.STRING)(str_annotated_2)
         )
